"""add max_steps to task v2 table

Revision ID: 07cb499ecbce
Revises: e8285b6ddcf0
Create Date: 2025-05-05 10:08:04.242755+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "07cb499ecbce"
down_revision: Union[str, None] = "e8285b6ddcf0"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("max_steps", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_cruises", "max_steps")
    # ### end Alembic commands ###
