"""add_include_action_history_in_verification_to_tasks

Revision ID: d90177b2b935
Revises: 11b84609ef08
Create Date: 2025-05-15 15:07:22.825001+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d90177b2b935"
down_revision: Union[str, None] = "11b84609ef08"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("tasks", sa.Column("include_action_history_in_verification", sa.Boolean(), nullable=True))
    op.add_column(
        "workflow_run_blocks", sa.Column("include_action_history_in_verification", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_run_blocks", "include_action_history_in_verification")
    op.drop_column("tasks", "include_action_history_in_verification")
    # ### end Alembic commands ###
